import React from 'react';
import { BrowserRouter as Router, Link, Route, Routes } from 'react-router-dom';

import EntityOsm from './pages/entity-osm';
import Header from './components/header';
import Home from './pages/home';

import packageInfo from '../package.json'

import config from './utils/config';

const VERSION = packageInfo.version;
const area = '(area);out body center qt;';

export default function App(){
    return (
	<Router>
	    <Header />
	    <p>
		This website (Version:&nbsp;{VERSION}) dislays public transport related information in the RVB area.
	    </p>
	    <h1>RVB Display</h1>
		<Routes>
		    <Route path='/' element={<Home/>}/>
		    <Route path='/bike-ride' element={<EntityOsm
							  address={config.ADDRESS + 'nwr[amenity=bicycle_parking][bike_ride]["bike_ride"!="no"]' + config.AREA }
							  title='B+R Stations'/>}/>
		    <Route path='/bus-stop' element={<EntityOsm
							 address={ config.ADDRESS + 'nwr[highway=bus_stop]' + config.AREA }
							  title='Bus Stops'/>}/>
		    <Route path='/park-ride' element={<EntityOsm
							  address={ config.ADDRESS + 'nwr["park_ride"!="no"]["park_ride"]' + config.AREA }
							  title='P+R Stations'/>}/>
		    <Route path='/taxi' element={<EntityOsm
						     address={ config.ADDRESS + 'nwr[amenity=taxi]' + config.AREA }
							  title='Taxi Stations'/>}/>
		    <Route path='/ticket-machine' element={<EntityOsm
							       address={ config.ADDRESS + 'nwr[amenity=vending_machine][vending=public_transport_tickets]' + config.AREA }
							       title='Ticket Machines'/>}/>
		    <Route path='/ticket-office' element={<EntityOsm
							      address={ config.ADDRESS + 'nwr[shop=ticket]["tickets:public_transport"!=no]' + config.AREA }
							      title='Ticket Offices'/>}/>
		    <Route path='/train-station' element={<EntityOsm
							      address={ config.ADDRESS + 'nwr[railway=station][usage!=tourism]' + config.AREA }
							      title='Train Stations'/>}/>
		    <Route path='/train-halt' element={<EntityOsm
							      address={ config.ADDRESS + 'nwr[railway=halt][usage!=tourism]' + config.AREA }
							      title='Train Halts'/>}/>
		</Routes>
	    </Router>
    );
}
