import React from 'react';
import PropTypes from 'prop-types';
import {MapContainer,Polyline,TileLayer} from 'react-leaflet';

/*JS module import (vs cdn or style link)*/
import 'leaflet/dist/leaflet.css'
import './entities-map.css';

import EntitiesMarker from './entities-marker';

export default function EntitiesMap({entities}) {
    /*lat and lon of Braunschweig,DE*/
    //const position = [52.26594, 10.52673];
    /*center map in the middle of RVB*/
    const position = [52.16594, 10.52673];
    return (
	<>
	    <MapContainer
		center={position}
		zoom={9}
		minZoom={4}
		maxZoom={18}
		maxBounds={[[-60, -180], [85, 180]]}
		scrollWheelZoom={true}
	    >

		<TileLayer
		    attribution='&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
		    url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"
		/>

		{
		    entities.map(function(value,key) {
			if(value[4]!==undefined && value[4]!==null && value[3]!==undefined && value[3]!==null){
			return <EntitiesMarker
				   key={value[0]}
				   index={value[0]}
				   id={value[0]}
				   lat={value[4]}
				   lon={value[3]}
				   name={value[2]}
			       />;
			}
                    })
		}
	    </MapContainer>
	</>
    );
}
EntitiesMap.propTypes = {
    entities: PropTypes.array
};
